`BestReg` <-
function(Xy.df, Criterion="UBIC", verbose=TRUE){
 stopifnot(is.data.frame(Xy.df), ncol(Xy.df)>1, length(names(Xy.df))==ncol(Xy.df))
 method<-Criterion
 if (is.na(pmatch(method,c("UBIC","AIC","BIC"))))
    method<-"UBIC"
 y<-Xy.df[,ncol(Xy.df)]
 X<-Xy.df[,-ncol(Xy.df)]
 n<-length(y)
 p<-ncol(X)
 #requires leaps package
 outLeaps <- leaps(y=y, x=X, nbest=1, method="r2", strictly.compatible=FALSE)
 #k will be the number of inputs + 1, for the mean
 k <- (outLeaps$size)  #k = 2, ..., p always!
 TotSS <- sum((y-mean(y))^2)
 #RSS is the residual sum of squares for the best model with k inputs
 RSS <- TotSS*(1-outLeaps$r2)
 LogL <- (-n/2)*log(RSS/n)
 LogL0 <- (-n/2)*log(TotSS/n)
 LogL <- c(LogL0, LogL)
 k <- c(1, k)
 if (method=="AIC")
    ic<- -2*LogL + 2*k
 if (method=="BIC")
    ic<- -2*LogL + log(n)*k
 if (method=="UBIC")
    ic<- -2*LogL + log(n)*k + 2*lchoose(p+1, k)
 ind<-order(ic)[1]
 if (verbose)
    message(paste("Best subset using ", method))
 if (ind == 1) {
        if (verbose)
            cat("No input variables selected.", fill=TRUE)
        yname<-names(Xy.df)[[ncol(Xy.df)]] 
        (fm<-as.formula(paste(yname, "~1")))
        lmobj<-lm(fm, data=Xy.df)
        }
 else   {
        ans<-outLeaps$which[ind-1,]  #vector of T/F indicating selection
        if (verbose)
            print(names(ans)[ans], fill=TRUE)
        lmobj<-lmXy(Xy.df[,c(ans,TRUE)])
        }
 lmobj
}

